--[[

  This is like Chronos, but without all the extra functions that noone really uses.
  Something Chronos can't, is schedule repeating timers.
  It works like this: scheduleRepeat(timername, time, function, args...)
  
  Whenever possible, use schedule instead of scheduleByName. It's faster!


  The used tables look like this:

  Timers = {
    {
      endtime,
      func,
      args,
    }
  }
  TimersByName = {
    name = {
      dt,
      endtime,
      func,
      args,
    }
  }
--]]


Timers = { n = 0 }
TimersByName = { }
TimersActive = false

function schedule(dt, func, ...)
  local index = Timers.n + 1
  if not Timers[index] then Timers[index] = { } end
  Timers[index].endtime = GetTime() + dt
  Timers[index].func = func
  Timers[index].args = arg
  Timers.n = index
  table.sort(Timers, function(t1, t2) return t2.endtime < t1.endtime end)
end

function scheduleByName(name, dt, func, ...)
  TimersByName[name] = {
    endtime = GetTime() + dt,
    func = func,
    args = arg
  }
end

function unscheduleByName(name)
  TimersByName[name] = nil
end

function isScheduledByName(name)
  return TimersByName[name]
end

function scheduleRepeat(name, dt, func, ...)
  TimersByName[name] = {
    dt = dt,
    endtime = GetTime() + dt,
    func = func,
    args = arg
  }
end

function timers_OnUpdate(dt)
  if not TimersActive then return end

  local now = GetTime()
  
  while Timers.n > 0 do
    local timer = Timers[Timers.n]
    if timer.endtime <= now then
      Timers.n = Timers.n - 1
      timer.func(unpack(timer.args))
    else
      break
    end
  end
  
  local name, timer, nextkey, nextvalue
  name, timer = next(TimersByName)
  while name ~= nil do
    nextkey, nextvalue = next(TimersByName, name)
    if timer.endtime <= now then
      timer.func(unpack(timer.args))
      if timer.dt then
        timer.endtime = now + timer.dt
      elseif timer == TimersByName[name] then
        TimersByName[name] = nil
      end
    end
    name, timer = nextkey, nextvalue
  end

end